@@ -1,5 +1,5 @@
 package URI::Fast;
-$URI::Fast::VERSION = '0.15';
+$URI::Fast::VERSION = '0.16';
 # ABSTRACT: A fast(er) URI parser
 
 use utf8;
@@ -7,14 +7,14 @@ use strict;
 use warnings;
 no strict 'refs';
 use Carp;
-require Inline;
-require Exporter;
-require File::Spec;
+use Inline;
+use Exporter;
+use File::Spec qw();
 
 # Build
 my $path = File::Spec->rel2abs(__FILE__);
 $path =~ s/\.pm$//;
-$path .= '/uri_fast.c';
+$path = File::Spec->catfile($path, 'uri_fast.c');
 Inline->bind(C => $path => optimize => '-O2');
 
 use parent 'Exporter';
@@ -145,7 +145,7 @@ URI::Fast - A fast(er) URI parser
 
 =head1 VERSION
 
-version 0.15
+version 0.16
 
 =head1 SYNOPSIS
 
